@@ -4,6 +4,7 @@
 #include "actions.h"
 #include "analyzer.h"
 #include "config.h"
+#include "configure.h"
 #include <cstdlib>
 #include "debug.h"
 #include "extern.h"       //dialog creation function definitions
@@ -13,6 +14,7 @@
 #include <kcursor.h>
 #include <kfiledialog.h>     //::open()
 #include <kglobalsettings.h> //::timerEvent()
+#include <kio/netaccess.h>
 #include <ksqueezedtextlabel.h>
 #include <kstatusbar.h>
 #include <ktoolbar.h>
@@ -23,11 +25,14 @@
 #include "playlistFile.h"
 #include "mxcl.library.h"
 #include <qcstring.h>
+#include <qdesktopwidget.h>
 #include <qevent.h>      //::stateChanged()
 #include <qlayout.h>     //ctor
 #include <qpopupmenu.h>  //because XMLGUI is poorly designed
+#include <qobjectlist.h>
 #include "slider.h"
 #include "theStream.h"
+#include "volumeAction.h"
 #include "xineEngine.h"
 
 #ifndef NO_XTEST_EXTENSION
@@ -48,13 +53,13 @@ namespace Codeine {
 
 MainWindow::MainWindow()
       : KMainWindow()
-      , m_positionSlider( new Slider( Qt::Horizontal, this, 65535 ) )
+      , m_positionSlider( new Slider( this, 65535 ) )
       , m_timeLabel( new QLabel( " 0:00:00 ", this ) )
       , m_titleLabel( new KSqueezedTextLabel( this ) )
 {
    DEBUG_BLOCK
 
-   clearWFlags( WDestructiveClose ); //we don't want to be destructively closed
+   clearWFlags( WDestructiveClose ); //we are allocated on the stack
 
    kapp->setMainWidget( this );
 
@@ -62,6 +67,10 @@ MainWindow::MainWindow()
    setCentralWidget( videoWindow() );
    setFocusProxy( videoWindow() ); // essential! See VideoWindow::event(), QEvent::FocusOut
 
+   // these have no affect beccause "KDE Knows Best" FFS
+   setDockEnabled( toolBar(), Qt::DockRight, false ); //doesn't make sense due to our large horizontal slider
+   setDockEnabled( toolBar(), Qt::DockLeft, false ); //as above
+
    m_titleLabel->setMargin( 2 );
    m_timeLabel->setFont( KGlobalSettings::fixedFont() );
    m_timeLabel->setAlignment( AlignCenter );
@@ -77,6 +86,29 @@ MainWindow::MainWindow()
    setupActions();
    setupGUI();
    setStandardToolBarMenuEnabled( false ); //bah to setupGUI()!
+   toolBar()->show(); //it's possible it would be hidden, but we don't want that as no UI way to show it!
+
+   // only show dvd button when playing a dvd
+   {
+      struct KdeIsTehSuck : public QObject
+      {
+         virtual bool eventFilter( QObject*, QEvent *e )
+         {
+            if (e->type() != QEvent::LayoutHint)
+               return false;
+
+            // basically, KDE shows all tool-buttons, even if they are
+            // hidden after it does any layout operation. Yay for KDE. Yay.
+            QWidget *button = (QWidget*)((KMainWindow*)mainWindow())->toolBar()->child( "toolbutton_toggle_dvd_menu" );
+            if (button)
+               button->setShown( TheStream::url().protocol() == "dvd" );
+            return false;
+         }
+      } *o;
+      o = new KdeIsTehSuck;
+      toolBar()->installEventFilter( o );
+      insertChild( o );
+   }
 
    {
       QPopupMenu *menu = 0, *settings = static_cast<QPopupMenu*>(factory()->container( "settings", this ));
@@ -101,7 +133,8 @@ MainWindow::MainWindow()
       settings->insertSeparator( index );
    }
 
-   if( !child( "dvdToolBar" ) ) {
+   QObjectList *list = toolBar()->queryList( "KToolBarButton" );
+   if (list->isEmpty()) {
       MessageBox::error( i18n(
             "<qt>" PRETTY_NAME " could not load its interface, this probably means that " PRETTY_NAME " is not "
             "installed to the correct prefix. If you installed from packages please contact the packager, if "
@@ -110,10 +143,12 @@ MainWindow::MainWindow()
 
       std::exit( 1 );
    }
+   delete list;
 
    KXMLGUIClient::stateChanged( "empty" );
 
-   if( KCmdLineArgs::parsedArgs()->count() || kapp->isRestored() )
+   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
+   if( args->count() || args->isSet( "play-dvd" ) || kapp->isRestored() )
       //we need to resize the window, so we can't show the window yet
       init();
    else {
@@ -141,7 +176,7 @@ MainWindow::init()
       std::exit( 2 );
    }
 
-   //would be dangerous for these to happen before the videoWindow() is initialised
+   //would be dangerous for these to65535 happen before the videoWindow() is initialised
    setAcceptDrops( true );
    connect( m_positionSlider, SIGNAL(sliderReleased( uint )), engine(), SLOT(seek( uint )) );
    connect( statusBar(), SIGNAL(messageChanged( const QString& )), engine(), SLOT(showOSD( const QString& )) );
@@ -150,7 +185,9 @@ MainWindow::init()
 
    if( !kapp->isRestored() ) {
       KCmdLineArgs &args = *KCmdLineArgs::parsedArgs();
-      if( args.count() > 0 ) {
+      if (args.isSet( "play-dvd" ))
+         open( "dvd:/" );
+      else if (args.count() > 0 ) {
          open( args.url( 0 ) );
          args.clear();
          adjustSize(); //will resize us to reflect the videoWindow's sizeHint()
@@ -171,9 +208,9 @@ MainWindow::~MainWindow()
 {
    DEBUG_FUNC_INFO
 
-   hide();
+   hide(); //so we appear to have quit, and then sound fades out below
 
-   delete videoWindow();
+   delete videoWindow(); //fades out sound in dtor
 }
 
 bool
@@ -188,7 +225,7 @@ MainWindow::queryExit()
       // otherwise KMainWindow saves the screensize as maximised
       Codeine::MessageBox::sorry(
             "This annoying messagebox is to get round a bug in either KDE or Qt. "
-            "Sorry dude! I'll fix the bug as soon as I can figure out how..." );
+            "Just press OK and Codeine will quit." );
       //NOTE not actually needed
       saveAutoSaveSettings();
       hide();
@@ -211,17 +248,19 @@ MainWindow::setupActions()
    new PlayAction( this, SLOT(play()), ac );
    new KAction( i18n("Stop"), "player_stop", Key_S, engine(), SLOT(stop()), ac, "stop" );
 
-   new KToggleAction( i18n("Record"), "player_record", Key_R, engine(), SLOT(record()), ac, "record" );
+   new KToggleAction( i18n("Record"), "player_record", CTRL + Key_R, engine(), SLOT(record()), ac, "record" );
 
    new KAction( i18n("Reset Video Scale"), "viewmag1", Key_Equal, videoWindow(), SLOT(resetZoom()), ac, "reset_zoom" );
    new KAction( i18n("Media Information"), "messagebox_info", Key_I, this, SLOT(streamInformation()), ac, "information" );
-   new KToggleAction( i18n("Menu Toggle"), "dvd_unmount", Key_R, engine(), SLOT(toggleDVDMenu()), ac, "toggle_dvd_menu" );
+   new KAction( i18n("Menu Toggle"), "dvd_unmount", Key_R, engine(), SLOT(toggleDVDMenu()), ac, "toggle_dvd_menu" );
    new KAction( i18n("&Capture Frame"), "frame_image", Key_C, this, SLOT(captureFrame()), ac, "capture_frame" );
 
    new KAction( i18n("Video Settings..."), "configure", Key_V, this, SLOT(configure()), ac, "video_settings" );
    new KAction( i18n("Configure xine..."), "configure", 0, this, SLOT(configure()), ac, "xine_settings" );
 
    (new KWidgetAction( m_positionSlider, i18n("Position Slider"), 0, 0, 0, ac, "position_slider" ))->setAutoSized( true );
+
+   new VolumeAction( toolBar(), ac );
 }
 
 void
@@ -296,6 +335,9 @@ MainWindow::engineMessage( const QString &message )
 bool
 MainWindow::open( const KURL &url )
 {
+   DEBUG_BLOCK
+   debug() << url << endl;
+
    if( load( url ) ) {
       const int offset = TheStream::hasProfile()
             // adjust offset if we have session history for this video
@@ -331,6 +373,19 @@ MainWindow::load( const KURL &url )
       }
    }
 
+   if (url.protocol() == "media") {
+      #define UDS_LOCAL_PATH (72 | KIO::UDS_STRING)
+      KIO::UDSEntry e;
+      if (!KIO::NetAccess::stat( url, e, 0 ))
+         MessageBox::sorry( "There was an internal error with the media slave..." );
+      else {
+         KIO::UDSEntry::ConstIterator end = e.end();
+         for (KIO::UDSEntry::ConstIterator it = e.begin(); it != end; ++it)
+            if ((*it).m_uds == UDS_LOCAL_PATH && !(*it).m_str.isEmpty())
+               return engine()->load( KURL::fromPathOrURL( (*it).m_str ) );
+      }
+   }
+
    //let xine handle invalid, etc, KURLS
    //TODO it handles non-existant files with bad error message
    return engine()->load( url );
@@ -374,7 +429,7 @@ MainWindow::playMedia( bool show_welcome_dialog )
       open( "cdda:/1" );
       break;
    case PlayDialog::VCD:
-      open( "vcd:/" );
+      open( "vcd://" ); // one / is not enough
       break;
    case PlayDialog::DVD:
       open( "dvd:/" );
@@ -382,9 +437,112 @@ MainWindow::playMedia( bool show_welcome_dialog )
    }
 }
 
+class FullScreenToolBarHandler : QObject
+{
+   KToolBar *m_toolbar;
+   int m_timer_id;
+   bool m_stay_hidden_for_a_bit;
+   QPoint m_home;
+
+public:
+   FullScreenToolBarHandler( KMainWindow *parent )
+         : QObject( parent )
+         , m_toolbar( parent->toolBar() )
+         , m_timer_id( 0 )
+         , m_stay_hidden_for_a_bit( false )
+   {
+      DEBUG_BLOCK
+
+      parent->installEventFilter( this );
+      m_toolbar->installEventFilter( this );
+   }
+
+   bool eventFilter( QObject *o, QEvent *e )
+   {
+      if (o == parent() && e->type() == QEvent::MouseMove) {
+         killTimer( m_timer_id );
+
+         QMouseEvent const * const me = (QMouseEvent*)e;
+         if (m_stay_hidden_for_a_bit) {
+            // wait for a small pause before showing the toolbar again
+            // usage = user removes mouse from toolbar after using it
+            // toolbar disappears (usage is over) but usually we show
+            // toolbar immediately when mouse is moved.. so we need this hack
+
+            // HACK if user thrusts mouse to top, we assume they really want the toolbar
+            // back. Is hack as 80% of users have at top, but 20% at bottom, we don't cater
+            // for the 20% as lots more code, for now.
+            if (me->pos().y() < m_toolbar->height())
+               goto show_toolbar;
+
+            m_timer_id = startTimer( 100 );
+         }
+         else {
+            if (m_toolbar->isHidden()) {
+               if (m_home.isNull())
+                  m_home = me->pos();
+               else if ((m_home - me->pos()).manhattanLength() > 6)
+                  // then cursor has moved far enough to trigger show toolbar
+show_toolbar:
+                  m_toolbar->show(),
+                  m_home = QPoint();
+               else
+                  // cursor hasn't moved far enough yet
+                  // don't reset timer below, return instead
+                  return false;
+            }
+
+            // reset the hide timer
+            m_timer_id = startTimer( VideoWindow::CURSOR_HIDE_TIMEOUT );
+         }
+      }
+
+      if (o == parent() && e->type() == QEvent::Resize)
+      {
+         //we aren't managed by mainWindow when at FullScreen
+         videoWindow()->move( 0, 0 );
+         videoWindow()->resize( ((QWidget*)o)->size() );
+         videoWindow()->lower();
+      }
+
+      if (o == m_toolbar)
+         switch (e->type()) {
+            case QEvent::Enter:
+               m_stay_hidden_for_a_bit = false;
+               killTimer( m_timer_id );
+            break;
+
+            case QEvent::Leave:
+               m_toolbar->hide();
+               m_stay_hidden_for_a_bit = true;
+               killTimer( m_timer_id );
+               m_timer_id = startTimer( 100 );
+            break;
+
+            default: break;
+         }
+
+      return false;
+   }
+
+   void timerEvent( QTimerEvent* )
+   {
+      if (m_stay_hidden_for_a_bit)
+         ;
+
+      else if (!m_toolbar->hasMouse())
+         m_toolbar->hide();
+
+      m_stay_hidden_for_a_bit = false;
+   }
+};
+
+
 void
 MainWindow::fullScreenToggled( bool isFullScreen )
 {
+   static FullScreenToolBarHandler *s_handler;
+
    DEBUG_FUNC_INFO
 
    if( isFullScreen )
@@ -397,12 +555,18 @@ MainWindow::fullScreenToggled( bool isFullScreen )
    toolBar()->setMovingEnabled( !isFullScreen );
    toolBar()->setHidden( isFullScreen && engine()->state() == Engine::Playing );
 
-   toolBar( "dvdToolBar" )->setHidden( isFullScreen || TheStream::url().protocol() != "dvd" );
-
    reinterpret_cast<QWidget*>(menuBar())->setHidden( isFullScreen );
    statusBar()->setHidden( isFullScreen );
 
    setMouseTracking( isFullScreen ); /// @see mouseMoveEvent()
+
+   if (isFullScreen)
+      s_handler = new FullScreenToolBarHandler( this );
+   else
+      delete s_handler;
+
+   // prevent videoWindow() moving around when mouse moves
+   setCentralWidget( isFullScreen ? 0 : videoWindow() );
 }
 
 void
@@ -443,7 +607,7 @@ MainWindow::setChannels( const QStringList &channels )
    //NOTE we subtract 2 in xineEngine because QMenuData doesn't allow negative id
    int id = 2;
    ++it;
-   for( QStringList::ConstIterator end = channels.end(); it != end; ++it, ++id )
+   for( QStringList::ConstIterator const end = channels.end(); it != end; ++it, ++id )
       menu->insertItem( *it, id );
 
    menu->insertSeparator();
@@ -511,13 +675,6 @@ MainWindow::keyPressEvent( QKeyEvent *e )
    #undef seek
 }
 
-void
-MainWindow::mouseMoveEvent( QMouseEvent *e )
-{
-   if( toggleAction( "fullscreen" )->isChecked() && engine()->state() == Engine::Playing )
-      toolBar()->setShown( e->pos().y() == 0 );
-}
-
 QPopupMenu*
 MainWindow::menu( const char *name )
 {
