@@ -16,15 +16,10 @@ namespace Codeine
       static Slider *instance() { return s_instance; }
 
    public:
-      Slider( Qt::Orientation, QWidget*, uint max = 0 );
+      Slider( QWidget*, uint max = 0 );
 
       virtual void setValue( int );
 
-      //WARNING non-virtual - and thus only really intended for internal use
-      //this is a major flaw in the class presently, however it suits our
-      //current needs fine
-      int value() const { return adjustValue( QSlider::value() ); }
-
    signals:
       //we emit this when the user has specifically changed the slider
       //so connect to it if valueChanged() is too generic
@@ -38,9 +33,10 @@ namespace Codeine
       virtual void mousePressEvent( QMouseEvent* );
       virtual void keyPressEvent( QKeyEvent *e ) { e->ignore(); } //so that MainWindow gets the keypress
 
-      bool m_sliding;
+      virtual QSize sizeHint() const { return QSlider::sizeHint() + QSize( 0, 6 ); }
+      virtual QSize minimumSizeHint() const { return sizeHint(); }
 
-      int adjustValue( int v ) const { return orientation() == Vertical ? maxValue() - v : v; }
+      bool m_sliding;
 
    private:
       static Slider *s_instance;
