@@ -69,7 +69,7 @@ VideoWindow::~VideoWindow()
       int cum = 0;
       for( int v = 99; v >= 0; v-- ) {
          xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, v );
-         int sleep = int(13000 * (-std::log10( double(v + 1) ) + 2));
+         int sleep = int(32000 * (-std::log10( double(v + 1) ) + 2));
 
          ::usleep( sleep );
 
@@ -252,6 +252,7 @@ VideoWindow::load( const KURL &url )
       setParameter( XINE_PARAM_SPU_CHANNEL, -1 );
       setParameter( XINE_PARAM_AUDIO_CHANNEL_LOGICAL, -1 );
       setParameter( XINE_PARAM_VO_ASPECT_RATIO, 0 );
+      setParameter( XINE_PARAM_AUDIO_AMP_LEVEL, 100 );
       #undef setParameter
 
       videoWindow()->setShown( xine_get_stream_info( m_stream, XINE_STREAM_INFO_HAS_VIDEO ) );
@@ -291,7 +292,7 @@ VideoWindow::play( uint offset )
    if( resume )
       //HACK because we have to do xine_play() the audio "stutters"
       //     so we mute it and then unmute it to make it sound better
-      xine_set_param( m_stream, XINE_PARAM_AUDIO_MUTE, 1 );
+      xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_MUTE, 1 );
 
    debug() << "xine_play()\n";
    if( xine_play( m_stream, offset, 0 ) )
@@ -304,7 +305,7 @@ VideoWindow::play( uint offset )
          pause();
 
          // see above from HACK
-         xine_set_param( m_stream, XINE_PARAM_AUDIO_MUTE, 0 );
+         xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_MUTE, 0 );
       }
       else
          announceStateChange();
@@ -323,7 +324,11 @@ VideoWindow::record()
 
    if( xine_config_lookup_entry( m_xine, "misc.save_dir", &config ) )
    {
-      config.str_value = "/home/mxcl/Desktop";
+      //TODO which fricking KDE function tells me this? Who can tell, stupid KDE API
+      QDir d( QDir::home().filePath( "Desktop" ) );
+      config.str_value = qstrdup( d.exists() //FIXME tiny-mem-leak, *shrug*
+            ? d.path().utf8()
+            : QDir::homeDirPath().utf8() );
       xine_config_update_entry( m_xine, &config );
 
       const QString fileName = m_url.filename();
@@ -436,6 +441,13 @@ VideoWindow::posTimeLength( PosTimeLength type ) const
    return 0; //--warning
 }
 
+uint
+VideoWindow::volume() const
+{
+   //TODO I don't like the design
+   return xine_get_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL );
+}
+
 void
 VideoWindow::seek( uint pos )
 {
@@ -466,7 +478,7 @@ VideoWindow::seek( uint pos )
       // xine_play unpauses stream if stream was paused
       // was broken at 1.0.1 still
       wasPaused = true;
-      xine_set_param( m_stream, XINE_PARAM_AUDIO_MUTE, 1 );
+      xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_MUTE, 1 );
       break;
    default:
       ;
@@ -516,7 +528,7 @@ VideoWindow::seek( uint pos )
 
    if( wasPaused )
       xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE ),
-      xine_set_param( m_stream, XINE_PARAM_AUDIO_MUTE, 0 );
+      xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_MUTE, 0 );
 }
 
 void
@@ -541,6 +553,8 @@ VideoWindow::setStreamParameter( int value )
       value -= 2;
    else if( sender == "aspect_ratio_menu" )
       parameter = XINE_PARAM_VO_ASPECT_RATIO;
+   else if( sender == "volume" )
+      parameter = XINE_PARAM_AUDIO_AMP_LEVEL;
    else
       return;
 
